program OwlStats;

{$R OWLSTATS.RES}

uses Objects, OWindows, ODialogs, OMemory, WinTypes, WinProcs,
  StatDLL, Strings, BWCC;

{$I OWLSTATS.INC}

type
  PStatDialog = ^TStatDialog;
  TStatDialog = object(TDlgWindow)
    AddButton: PButton;
    ClearButton: PButton;
    DeleteButton: PButton;
    SampleInput: PEdit;
    SampleList: PListBox;
    Mean: PStatic;
    Sum: PStatic;
    NumSamples: PStatic;
    Squares: PStatic;
    StandardDev: PStatic;
    PopulationDev: PStatic;
    CalcContext: TStatContext;
    constructor Init(AParent: PWindowsObject; AName: PChar);
    destructor Done; virtual;
    procedure AddItem(var Msg: TMessage); virtual
      id_First + idAddButton;
    procedure ClearSamples(var Msg: TMessage); virtual
      id_First + idClearButton;
    procedure DeleteItem(var Msg: TMessage); virtual
      id_First + idDeleteButton;
    function GetClassName: PChar; virtual;
    procedure GetWindowClass(var AWndClass: TWndClass); virtual;
    procedure InitControls;
    procedure NewCalc(Update: Boolean);
    procedure SetupWindow; virtual;
    function ShowError(ErrorCode: TStatResult): Boolean;
    procedure UpdateDisplay;
  end;

  PStatApplication = ^TStatApplication;
  TStatApplication = object(TApplication)
    procedure InitMainWindow; virtual;
  end;

procedure GetMem(var P: Pointer; Size: Word); export;
begin
  System.GetMem(P, Size);
end;

procedure FreeMem(var P: Pointer; Size: Word); export;
begin
  System.FreeMem(P, Size);
end;

constructor TStatDialog.Init(AParent: PWindowsObject; AName: PChar);
begin
  inherited Init(AParent, AName);
  CalcContext := nil;
  InitControls;
  NewCalc(False);
end;

destructor TStatDialog.Done;
begin
  if CalcContext <> nil then
    DoneCalc(CalcContext);
  inherited Done;
end;

procedure TStatDialog.AddItem(var Msg: TMessage);
var
  ASample: TSample;
  InputVal: array[0..20] of Char;
  Code: Integer;
begin
  SampleInput^.GetText(InputVal, SizeOf(InputVal));
  Val(InputVal, ASample.Sample, Code);
  if ShowError(AddSample(CalcContext, ASample)) then
    UpdateDisplay;
  SampleInput^.SetSelection(0, 20);
  SampleInput^.Focus;
end;

procedure TStatDialog.ClearSamples(var Msg: TMessage);
begin
  NewCalc(True);
end;

procedure TStatDialog.DeleteItem(var Msg: TMessage);
var
  SampleNum: Integer;
begin
  SampleNum := SampleList^.GetSelIndex;
  if SampleNum < 0 then Exit;
  if ShowError(DeleteSample(CalcContext, SampleNum)) then
    UpdateDisplay;
end;

function TStatDialog.GetClassName: PChar;
begin
  GetClassName := 'BorDlg_StatDlg';
end;

procedure TStatDialog.GetWindowClass(var AWndClass: TWndClass);
begin
  inherited GetWindowClass(AWndClass);
  AWndClass.hIcon := LoadIcon(HInstance, 'StatIcon');
end;

procedure TStatDialog.InitControls;
begin
  AddButton := New(PButton, InitResource(@Self, idAddButton));
  DeleteButton := New(PButton, InitResource(@Self, idDeleteButton));
  ClearButton := New(PButton, InitResource(@Self, idClearButton));
  SampleInput := New(PEdit, InitResource(@Self, idSampleInput, 20));
  SampleList := New(PListBox, InitResource(@Self, idSampleList));
  Mean := New(PStatic, InitResource(@Self, idMean, 20));
  Sum := New(Pstatic, InitResource(@Self, idSum, 20));
  NumSamples := New(PStatic, InitResource(@Self, idSampleCount, 20));
  Squares := New(PStatic, InitResource(@Self, idSquares, 20));
  StandardDev := New(PStatic, InitResource(@Self, idStandardDev, 20));
  PopulationDev := New(PStatic, InitResource(@Self, idPopulationDev, 20));
end;

procedure TStatDialog.NewCalc(Update: Boolean);
begin
  if CalcContext <> nil then DoneCalc(CalcContext);
  CalcContext := InitCalc(10, 10, GetMem, FreeMem);
  if Update then UpdateDisplay;
end;

procedure TStatDialog.SetupWindow;
begin
  inherited SetupWindow;
  UpdateDisplay;
end;

function TStatDialog.ShowError(ErrorCode: TStatResult): Boolean;
var
  MsgStr: array[0..80] of Char;
begin
  ShowError := True;
  if ErrorCode = srOK then Exit
  else ShowError := False;
  case ErrorCode of
    srInvalidContext: StrCopy(MsgStr, 'Calculation context is invalid');
    srNoData: StrCopy(MsgStr, 'No data for calculations');
    srNoMemory: StrCopy(MsgStr, 'Out of memory');
    srMaxSamples: StrCopy(MsgStr, 'Sample limit reached');
    srInvalidSample: StrCopy(MsgStr, 'Sample is invalid');
    srError: StrCopy(MsgStr, 'STAT.DLL Error');
  end;
  MessageBox(HWindow, MsgStr, 'Calculation error', mb_OK or mb_IconExclamation);
end;

procedure TStatDialog.UpdateDisplay;
const
  ErrString = 'Undefined';
var
  I, Count: Integer;
  SampleStr: array[0..20] of Char;
  ASample: PSample;

  procedure UpdateControl(AStatic: PStatic; Calc: TCalcResult; ErrStr: PChar);
  var
    CalcResult: Real;
    ResultStr: array[0..20] of Char;
  begin
    if GetCalc(CalcContext, Calc, CalcResult) = srOK then
      Str(CalcResult:20:10, ResultStr)
    else StrCopy(ResultStr, ErrStr);
    AStatic^.SetText(ResultStr);
  end;

begin
  SampleList^.ClearList;
  if ShowError(SampleCount(CalcContext, Count)) then
  begin
    Str(Count, SampleStr);
    NumSamples^.SetText(SampleStr);
    if Count > 0 then
    begin
      for I := 0 to Count - 1 do
      begin
        if not ShowError(GetSample(CalcContext, I, ASample)) then
          Break;
        Str(ASample^.Sample:20:10, SampleStr);
        SampleList^.AddString(SampleStr);
      end;
    end;
    UpdateControl(Mean, crMean, ErrString);
    UpdateControl(Sum, crSum, ErrString);
    UpdateControl(Squares, crSumOfSquares, ErrString);
    UpdateControl(StandardDev, crStandard, ErrString);
    UpdateControl(PopulationDev, crPopulation, ErrString);
  end;
end;

procedure TStatApplication.InitMainWindow;
begin
  MainWindow := New(PStatDialog, Init(nil, 'StatDlg'));
end;

var
  StatApplication: TStatApplication;

begin
  StatApplication.Init('OWL Stats');
  StatApplication.Run;
  StatApplication.Done;
end.
