program TVStats;

uses Objects, Drivers, Views, Menus, Dialogs, MsgBox, App, StatDLL;

const
  cmOpenStatWindow = 1000;
  cmAddSample      = 1001;
  cmDeleteSample   = 1002;
  cmClearSamples   = 1003;
  cmStatWindow     = 1004;

  CStatText        = #19;

type
  PSampleList = ^TSampleList;
  TSampleList = object(TListViewer)
    CalcContext: TStatContext;
    constructor Init(var Bounds: TRect; ANumCols: Integer; AHScrollBar,
      AVScrollBar: PScrollBar; ACalcContext: TStatContext);
    function GetText(Item: Integer; MaxLen: Integer): String; virtual;
  end;

  PStatApp = ^TStatApp;
  TStatApp = object(TApplication)
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure InitDeskTop; virtual;
    procedure InitMenuBar; virtual;
    procedure InitStatusLine; virtual;
    procedure NewStatWindow;
  end;

  PStatText = ^TStatText;
  TStatText = object(TStaticText)
    Undefined: Boolean;
    function GetPalette: PPalette; virtual;
    procedure SetUndefined;
  end;

  PRealStatText = ^TRealStatText;
  TRealStatText = object(TStatText)
    Value: Real;
    constructor Init(var Bounds: TRect; AValue: Real);
    function DataSize: Word; virtual;
    procedure GetData(var Rec); virtual;
    procedure GetText(var S: String); virtual;
    procedure SetData(var Rec); virtual;
  end;

  PIntStatText = ^TIntStatText;
  TIntStatText = object(TStatText)
    Value: Integer;
    constructor Init(var Bounds: TRect; AValue: Integer);
    function DataSize: Word; virtual;
    procedure GetData(var Rec); virtual;
    procedure GetText(var S: String); virtual;
    procedure SetData(var Rec); virtual;
  end;

  PStatWindow = ^TStatWindow;
  TStatWindow = object(TDialog)
    CalcContext: TStatContext;
    SampleInput: PInputLine;
    SampleList: PListViewer;
    Mean: PStatText;
    NumSamples: PStatText;
    PopulationDev: PStatText;
    Sum: PStatText;
    Squares: PStatText;
    StandardDev: PStatText;
    constructor Init(var Bounds: TRect; ATitle: TTitleStr);
    destructor Done; virtual;
    procedure SampleAdd;
    procedure SampleDelete;
    procedure ClearSamples;
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure UpdateStats;
  end;

procedure GetMem(var P: Pointer; Size: Word); export;
begin
  System.GetMem(P, Size);
end;

procedure FreeMem(var P: Pointer; Size: Word); export;
begin
  System.FreeMem(P, Size);
end;

function ShowError(ErrorCode: TStatResult): Boolean;
var
  MsgStr: String;
begin
  ShowError := True;
  if ErrorCode = srOK then Exit
  else ShowError := False;
  case ErrorCode of
    srInvalidContext: MsgStr := 'Calculation context is invalid';
    srNoData: MsgStr := 'No data for calculations';
    srNoMemory: MsgStr := 'Out of memory';
    srMaxSamples: MsgStr := 'Sample limit reached';
    srInvalidSample: MsgStr := 'Sample is invalid';
    srError: MsgStr := 'STAT.DLL Error';
  end;
  MessageBox(MsgStr, nil, mfInformation + mfOKButton);
end;

{ TSampleList }

constructor TSampleList.Init(var Bounds: TRect; ANumCols: Integer; AHScrollBar,
  AVScrollBar: PScrollBar; ACalcContext: TStatContext);
begin
  inherited Init(Bounds, ANumCols, AHScrollBar, AVScrollBar);
  CalcContext := ACalcContext;
end;

function TSampleList.GetText(Item: Integer; MaxLen: Integer): String;
var
  ValStr: String[20];
  ASample: PSample;
begin
  if ShowError(GetSample(CalcContext, Item, ASample)) then
  begin
    Str(ASample^.Sample:18:9, ValStr);
    GetText := ValStr;
  end else GetText := '';
end;

{ TStatText }

function TStatText.GetPalette: PPalette;
const
  P: String[Length(CStatText)] = CStatText;
begin
  GetPalette := @P;
end;

procedure TStatText.SetUndefined;
begin
  Undefined := True;
  DrawView;
end;

{ TRealStatText }

constructor TRealStatText.Init(var Bounds: TRect; AValue: Real);
begin
  inherited Init(Bounds, '');
  Value := AValue;
end;

function TRealStatText.DataSize: Word;
begin
  DataSize := SizeOf(Real);
end;

procedure TRealStatText.GetData(var Rec);
begin
  Real(Rec) := Value;
end;

procedure TRealStatText.GetText(var S: String);
begin
  if Undefined then S := '         Undefined'
  else Str(Value:18:9, S);
end;

procedure TRealStatText.SetData(var Rec);
begin
  Value := Real(Rec);
  Undefined := False;
  DrawView;
end;

{ TIntStatText }

constructor TIntStatText.Init(var Bounds: TRect; AValue: Integer);
begin
  inherited Init(Bounds, '');
  Value := AValue;
end;

function TIntStatText.DataSize: Word;
begin
  DataSize := SizeOf(Integer);
end;

procedure TIntStatText.GetData(var Rec);
begin
  Integer(Rec) := Value;
end;

procedure TIntStatText.GetText(var S: String);
begin
  Str(Value:18, S);
end;

procedure TIntStatText.SetData(var Rec);
begin
  Value := Integer(Rec);
  Undefined := False;
  DrawView;
end;

{ TStatApp }

procedure TStatApp.HandleEvent(var Event: TEvent);
begin
  inherited HandleEvent(Event);
  if (Event.What = evCommand) and (Event.Command = cmOpenStatWindow) then
    NewStatWindow;
end;

procedure TStatApp.InitDeskTop;
var
  R: TRect;
begin
  GetExtent(R);
  Dec(R.B.Y);
  DeskTop := New(PDeskTop, Init(R));
end;

procedure TStatApp.InitMenuBar;
begin
end;

procedure TStatApp.InitStatusLine;
var
  R: TRect;
begin
  GetExtent(R);
  R.A.Y := R.B.Y - 1;
  StatusLine := New(PStatusLine, Init(R,
    NewStatusDef(0, $FFFF,
      NewStatusKey('~Alt+X~ Exit', kbAltX, cmQuit,
      NewStatusKey('~F3~ New Window', kbF3, cmOpenStatWindow,
      NewStatusKey('~Alt+F3~ Close', kbAltF3, cmClose,
      NewStatusKey('~F6~ Next', kbF6, cmNext,
      NewStatusKey('~Shift+F6~ Prev', kbShiftF6, cmPrev,
      nil))))), nil)));
end;

procedure TStatApp.NewStatWindow;
const
  DialogX = 64;
  DialogY = 15;
var
  Dlg: PStatWindow;
  Control: PView;
  R, D: TRect;
begin
  Dlg := Message(@Self, evBroadCast, cmStatWindow, nil);
  if Dlg <> nil then with Dlg^ do
  begin
    Desktop^.GetExtent(D);
    R.A := Origin;
    R.B.X := R.A.X + DialogX;
    R.B.Y := R.A.Y + DialogY;
    if R.B.Y <> D.B.Y then R.Move(0, 1);
    if R.B.X <> D.B.X then R.Move(1, 0);
  end else R.Assign(0, 0, DialogX, DialogY);
  Dlg := New(PStatWindow, Init(R, 'Statistics'));
  with Dlg^ do
  begin
    R.Assign(2, 3, 22, 4);
    SampleInput := New(PInputLine, Init(R, 20));
    Insert(SampleInput);
    R.Assign(1, 2, 22, 3);
    Insert(New(PLabel, Init(R, '~S~ample', SampleInput)));
    R.Assign(21, 5, 22, 13);
    Control := New(PScrollBar, Init(R));
    Insert(Control);
    R.Assign(2, 5, 21, 13);
    SampleList := New(PSampleList, Init(R, 1, nil, PScrollBar(Control),
      CalcContext));
    Insert(SampleList);
    R.Assign(24, 3, 42, 4);
    NumSamples := New(PIntStatText, Init(R, 0));
    Insert(NumSamples);
    R.Assign(23, 2, 42, 3);
    Insert(New(PLabel, Init(R, 'Number of Samples', NumSamples)));
    R.Assign(24, 6, 42, 7);
    Mean := New(PRealStatText, Init(R, 0.0));
    Mean^.SetUndefined;
    Insert(Mean);
    R.Assign(23, 5, 42, 6);
    Insert(New(PLabel, Init(R, 'Mean', NumSamples)));
    R.Assign(24, 9, 42, 10);
    Sum := New(PRealStatText, Init(R, 0.0));
    Sum^.SetUndefined;
    Insert(Sum);
    R.Assign(23, 8, 42, 9);
    Insert(New(PLabel, Init(R, 'Sum', Sum)));
    R.Assign(44, 3, 62, 4);
    Squares := New(PRealStatText, Init(R, 0.0));
    Squares^.SetUndefined;
    Insert(Squares);
    R.Assign(43, 2, 62, 3);
    Insert(New(PLabel, Init(R, 'Sum of squares', Squares)));
    R.Assign(44, 6, 62, 7);
    StandardDev := New(PRealStatText, Init(R, 0.0));
    StandardDev^.SetUndefined;
    Insert(StandardDev);
    R.Assign(43, 5, 62, 6);
    Insert(New(PLabel, Init(R, 'Standard dev.', NumSamples)));
    R.Assign(44, 9, 62, 10);
    PopulationDev := New(PRealStatText, Init(R, 0.0));
    PopulationDev^.SetUndefined;
    Insert(PopulationDev);
    R.Assign(43, 8, 62, 9);
    Insert(New(PLabel, Init(R, 'Population dev.', NumSamples)));
    R.Assign(24, 12, 36, 14);
    Insert(New(PButton, Init(R, '~A~dd', cmAddSample, bfDefault)));
    Inc(R.A.X, 13); Inc(R.B.X, 13);
    Insert(New(PButton, Init(R, '~D~elete', cmDeleteSample, bfNormal)));
    Inc(R.A.X, 13); Inc(R.B.X, 13);
    Insert(New(PButton, Init(R, '~C~lear', cmClearSamples, bfNormal)));
    SelectNext(False);
  end;
  Desktop^.Insert(Dlg);
end;

{ TStatWindow }

constructor TStatWindow.Init(var Bounds: TRect; ATitle: TTitleStr);
begin
  inherited Init(Bounds, ATitle);
  CalcContext := InitCalc(10, 10, GetMem, FreeMem);
end;

destructor TStatWindow.Done;
begin
  if CalcContext <> nil then DoneCalc(CalcContext);
  inherited Done;
end;

procedure TStatWindow.SampleAdd;
var
  ASample: TSample;
  ValStr: String[20];
  Code: Integer;
begin
  SampleInput^.GetData(ValStr);
  if ValStr <> '' then
  begin
    Val(ValStr, ASample.Sample, Code);
    if (Code = 0) and ShowError(AddSample(CalcContext, ASample)) then
    begin
      SampleCount(CalcContext, Code);
      with SampleList^ do
      begin
        SetRange(Code);
        DrawView;
      end;
      UpdateStats;
    end;
  end;
  SampleInput^.SelectAll(True);
  SampleInput^.Focus;
end;

procedure TStatWindow.SampleDelete;
var
  Count: Integer;
begin
  if not ShowError(SampleCount(CalcContext, Count)) then Exit;
  if Count > 0 then
    if ShowError(DeleteSample(CalcContext, SampleList^.Focused)) then
    begin
      if not ShowError(SampleCount(CalcContext, Count)) then Exit;
      with SampleList^ do
      begin
        SetRange(Count);
        if Focused > Range - 1 then Focused := Range - 1;
        DrawView;
      end;
      UpdateStats;
    end;
end;

procedure TStatWindow.ClearSamples;
var
  Count: Integer;
begin
  if not ShowError(SampleCount(CalcContext, Count)) then Exit;
  while Count > 0 do
  begin
    ShowError(DeleteSample(CalcContext, 0));
    if not (SampleCount(CalcContext, Count) = srOK) then Exit;
  end;
  if not ShowError(SampleCount(CalcContext, Count)) then Exit;
  SampleList^.SetRange(Count);
  UpdateStats;
end;

procedure TStatWindow.HandleEvent(var Event: TEvent);
begin
  inherited HandleEvent(Event);
  case Event.What of
    evBroadcast, evCommand:
      case Event.Command of
        cmStatWindow: ClearEvent(Event);
        cmAddSample: SampleAdd;
        cmDeleteSample: SampleDelete;
        cmClearSamples: ClearSamples;
      end;
  end;
end;

procedure TStatWindow.UpdateStats;
var
  Count: Integer;

  procedure UpdateControl(P: PStatText; Calc: TCalcResult);
  var
    Rslt: Real;
  begin
    if GetCalc(CalcContext, Calc, Rslt) = srOK then
      P^.SetData(Rslt) else P^.SetUndefined;
  end;

begin
  if ShowError(SampleCount(CalcContext, Count)) then
  begin
    NumSamples^.SetData(Count);
    UpdateControl(Mean, crMean);
    UpdateControl(Sum, crSum);
    UpdateControl(Squares, crSumOfSquares);
    UpdateControl(StandardDev, crStandard);
    UpdateControl(PopulationDev, crPopulation);
  end;
end;

var
  StatApp: TStatApp;

begin
  StatApp.Init;
  StatApp.Run;
  StatApp.Done;
end.

