unit StartUp;

interface

{$F+}

uses
   WinTypes, WinProcs, Strings, Win31;

function  AddToStartUp(cmd_line, caption, icon_file: PChar;
   icon_num: word): Bool;

implementation

const
   ddeml_fn = 'DDEML.DLL';
   ddeml_anchor: THandle = 0;

   cbf_Fail_AllSvrXActions = $0003f000;
   dmlErr_No_Error = 0;
   cp_WinAnsi = 1004;
   xclass_Flags = $4000;
   xtyp_Execute = $0050 or xclass_Flags;

type
  HConv = Longint;
  HSz = Longint;
  HDDEData = Longint;
  TCallback = function (CallType, Fmt: Word; Conv: HConv; hsz1, hsz2: HSZ;
    Data: HDDEData; Data1, Data2: Longint): HDDEData;

  PConvContext = ^TConvContext;
  TConvContext = record
    cb: Word;
    wFlags: Word;
    wCountryID: Word;
    iCodePage: Integer;
    dwLangID: Longint;
    dwSecurity: Longint;
  end;

var
   DDEInitialize: function(var inst: longint; callback: TCallback;
      cmd, res: longint): word;
   DDEUninitialize: function (inst: longint): Bool;
   DDEConnect: function (inst: longint; service, topic: HSZ;
      CC: PConvContext): HConv;
   DDEDisconnect: function (Conv: HConv): Bool;
   DDECreateStringHandle: function(inst: longint; sh: PChar;
      code_page: integer): HSZ;
   DDEFreeStringHandle: function (inst: longint; HSZ: HSZ): Bool;
   DDECreateDataHandle: function(inst: longint; src: Pointer;
      cb, off: longint; item: HSZ; fmt, cmd: word): HDDEData;
   DDEClientTransaction: function(Data: Pointer; DataLen: longint;
      conv: HConv; Item: HSZ; Fmt, DataType: word; timeout: longint;
      result: Plongint): HDDEData;

procedure FreeDDEML; forward;

{ ------------------------------------ }

function DDECallBack(wType, wFmt: word; hc: HConv; hsz1, hsz2: HSZ;
   ddedata: HDDEData; data1, data2: longint): HDDEData;

begin
   DDECallBack := 0;
end;

{ ------------------------------------ }

procedure InitDDEAnchors;

begin
   ddeml_anchor := 0;
   @DDEInitialize := nil;
   @DDECreateStringHandle := nil;
   @DDEConnect := nil;
   @DDEFreeStringHandle := nil;
   @DDECreateDataHandle := nil;
   @DDEClientTransaction := nil;
   @DDEDisconnect := nil;
   @DDEUninitialize := nil;
end;

{ ------------------------------------ }

function  LoadDDEML: boolean;

var
   old_error_setting: word;

   procedure FailedLoad;

   begin
      FreeDDEML;
      LoadDDEML := false;
   end;

begin
   if ddeml_anchor >= 32 then
      begin
         LoadDDEML := true;
         Exit;
      end;

   old_error_setting := SetErrorMode(SEM_NoOpenFileErrorBox);

   InitDDEAnchors;
   ddeml_anchor := LoadLibrary(ddeml_fn);

   if ddeml_anchor >= 32 then
      begin
         @DDEInitialize := GetProcAddress(ddeml_anchor,'DDEINITIALIZE');
         @DDECreateStringHandle := GetProcAddress(ddeml_anchor,
            'DDECREATESTRINGHANDLE');
         @DDEConnect := GetProcAddress(ddeml_anchor,'DDECONNECT');
         @DDEFreeStringHandle := GetProcAddress(ddeml_anchor,
            'DDEFREESTRINGHANDLE');
         @DDECreateDataHandle := GetProcAddress(ddeml_anchor,
            'DDECREATEDATAHANDLE');
         @DDEClientTransaction := GetProcAddress(ddeml_anchor,
            'DDECLIENTTRANSACTION');
         @DDEDisconnect := GetProcAddress(ddeml_anchor,'DDEDISCONNECT');
         @DDEUninitialize := GetProcAddress(ddeml_anchor,'DDEUNINITIALIZE');

         if (@DDEInitialize <> nil) and
            (@DDECreateStringHandle <> nil) and
            (@DDEConnect <> nil) and
            (@DDEFreeStringHandle <> nil) and
            (@DDECreateDataHandle <> nil) and
            (@DDEClientTransaction <> nil) and
            (@DDEDisconnect <> nil) and
            (@DDEUninitialize <> nil) then
            LoadDDEML := true
         else
            FailedLoad;
      end
   else
      FailedLoad;

   SetErrorMode(old_error_setting);
end;

{ ------------------------------------ }

procedure FreeDDEML;

begin
   if ddeml_anchor >= 32 then
      FreeLibrary(ddeml_anchor);

   InitDDEAnchors;
end;

{ ------------------------------------ }

function TellProgMan(cmd: PChar): Bool;

var
   dwDDEinst: longint;
   hszProgman: HSZ;
   hc: HConv;
   hExecData: HDDEData;

label
   Done;

begin
   TellProgMan := false;    { Premature exits are all failures, via Done }
   if not LoadDDEML then
      Goto Done;
   dwDDEinst := 0;
   if (DDEInitialize(dwDDEinst,DDECallback,CBF_FAIL_ALLSVRXACTIONS,
      0) <> DMLERR_NO_ERROR) then
      Goto Done;
   hszProgman := DDECreateStringHandle(dwDDEinst,'PROGMAN',
      CP_WINANSI);
   hc := DDEConnect(dwDDEinst,hszProgman,hszProgman,nil);
   DDEFreeStringHandle(dwDDEinst,hszProgman);

   if hc = 0 then
      Goto Done;

   hExecData := DDECreateDataHandle(dwDDEinst,cmd,
      StrLen(cmd)+1,0,0,0,0);
   DDEClientTransaction(PChar(hExecData),-1,hc,0,0,
      XTYP_EXECUTE,1000,nil);
   DDEDisconnect(hc);
   DDEUninitialize(dwDDEinst);
   TellProgMan := true;

Done:
   FreeDDEML;
end;

{ ------------------------------------ }

function CreateGroup(group: PChar): Bool;

var
   buff: array[0..255] of char;

begin
   wvsprintf(buff,'[CreateGroup(%s)]',group);
   Creategroup := TellProgMan(buff);
end;

{ ------------------------------------ }

function AddItem(cmd_line, caption, icon_file: PChar;
   icon_num: word): Bool;

var
   buff: array[0..255] of char;
   ptr_array: array[1..4] of PChar;

begin
   ptr_array[1] := cmd_Line;
   ptr_array[2] := caption;

   if (icon_file <> nil) and (StrLen(icon_file) > 0) then
      begin
         ptr_array[3] := icon_file;
         ptr_array[4] := PChar(icon_num);
         wvsprintf(buff,'[AddItem(%s,%s,%s,%u)]',ptr_array);
      end
   else
      wvsprintf(buff,'[AddItem(%s,%s)]',ptr_array);

   AddItem := TellProgMan(buff);
end;

{ ------------------------------------ }

function  AddToStartUp(cmd_line, caption, icon_file: PChar;
   icon_num: word): Bool;

begin
   if not CreateGroup('STARTUP') then
      AddToStartUp := false
   else
      AddToStartUp := AddItem(cmd_line,caption,icon_file,icon_num);
end;

{ ------------------------------------ }

end.
