// RSFILEAR.CPP

#include "rsfilear.h"

template <class ElementType>
RS_TFileArray<ElementType>::~RS_TFileArray()
{
  if (IsInitialized())
  {
#if defined(RS_BUILD_WIN16) || defined(RS_BUILD_MAC) FlushBuffer();
#endif
    FreeBuffer();
  }
}

template <class ElementType>
RS_TFileArray<ElementType>::RS_TFileArray(RS_CFile &theFile, long numberOfElements)
{
  fInitialized = FALSE;
  fFile = &theFile;
  // If a size was specified in constructor call, use
  // it, otherwise use file size.
  if (numberOfElements == -1)
    theFile.GetFileSize(fBufferSize);
  else
    fBufferSize = numberOfElements * sizeof (ElementType);
  if (AllocateBuffer(fBufferSize))
  {
#if defined(RS_BUILD_WIN16) || defined(RS_BUILD_MAC)
  if (LoadBuffer() != 0)
  {
     FreeBuffer();
     return;

  }
#endif
  }
  else
  {
     // Throw exception
      return;
  }
  fInitialized = TRUE;
}

#ifdef RS_BUILD_WIN32
template <class ElementType>
void RS_TFileArray<ElementType>::FreeBuffer()
{
  UnmapViewOfFile(fMemoryPointer);
}

template <class ElementType>
int RS_TFileArray<ElementType>::AllocateBuffer(long size)
{
  DWORD desiredAccess = PAGE_READWRITE;
  char mapName[] = "";
  HANDLE theFile = (HANDLE) fFile->GetHFile();
  fHandle = CreateFileMapping(theFile, (LPSECURITY_ATTRIBUTES) NULL, desiredAccess, 0, 0, mapName);
  if (!fHandle)
    return FALSE;
  fMemoryPointer = MapViewOfFile(fHandle, FILE_MAP_WRITE, 0, 0, size);
  if (!fMemoryPointer)
    return FALSE;
  return TRUE;
}
#endif
#if defined(RS_BUILD_WIN16) || defined(RS_BUILD_MAC)
// To keep the code short, we just use new and delete
// for 16-bit Windows and Macintosh. We really should
// be using GlobalAlloc for 16-bit Windows to allow
// for > 64K buffers.
template <class ElementType>
void RS_TFileArray<ElementType>::FreeBuffer()
{
  delete [] (char *) fMemoryPointer;
}

template <class ElementType>
int RS_TFileArray<ElementType>::AllocateBuffer(long size)
{
  fMemoryPointer = (LPVOID) new char[size];
  return fMemoryPointer != NULL;
}
#endif

#if defined(RS_BUILD_WIN16) || defined(RS_BUILD_MAC)
template <class ElementType>
int RS_TFileArray<ElementType>::LoadBuffer()
{
  fFile->SetFilePosition(0);
  long bufferSize = fBufferSize;
  return fFile->Read(bufferSize, fMemoryPointer);
}
template <class ElementType>
void RS_TFileArray<ElementType>::FlushBuffer()
{
  fFile->SetFilePosition(0);
  long bufferSize = fBufferSize;
  fFile->Write(bufferSize, fMemoryPointer);
}
#endif


