//  RSFILES.CPP

#ifndef __RSFILES_H
#include "rsfiles.h"
#endif

#ifdef RS_BUILD_WIN
RS_CFile::RS_CFile(LPCSTR fileName, int accessMode)
{
  fFile = kFileError;
  HFILE fileHandle = _lopen(fileName, accessMode);
  if (fileHandle == HFILE_ERROR)
  {
    fileHandle = _lcreat(fileName, 0);
    if (fileHandle == HFILE_ERROR)
      return;
  }
  fFile = fileHandle;
}

RS_CFile::FileError RS_CFile::Read(long &count, void *buffer)
{
  long actualCount = _hread(fFile, (void _huge *) buffer, count);
  if (actualCount == -1L)
    return actualCount;
  else
  {
    count = actualCount;
    return kNoError;
  }
}

RS_CFile::FileError RS_CFile::Write(long &count, void *buffer)
{
  long actualCount = _hwrite(fFile, (void _huge *) buffer, count);
  if (actualCount == -1L)
    return actualCount;
  else
  {
    count = actualCount;
    return kNoError;
  }
}

RS_CFile::FileError RS_CFile::GetFileSize(long &fileSize)
{
  long filePos = _llseek(fFile, 0L, 2);
  if (filePos == HFILE_ERROR)
  {
    fileSize = 0;
    return filePos;
  }
  fileSize = filePos;
  return kNoError;
}

RS_CFile::FileError RS_CFile::SetFilePosition(long position)
{
  if (_llseek(fFile, position, 0) != HFILE_ERROR)
    return kNoError;
  else
    return kFileError;
}

RS_CFile::FileError RS_CFile::GetFilePosition(long &position)
{
  if (_llseek(fFile, 0L, 1) != HFILE_ERROR)
    return kNoError;
  else
    return kFileError;
}

RS_CFile::FileError RS_CFile::Close()
{
  return _lclose(fFile);
}
#endif

#ifdef RS_BUILD_MAC
RS_CFile::RS_CFile(FSSpec &fileSpec) : fFSSpec(fileSpec)
{
  FileError error;
  fRefNum = kFileError;
  error = FSpOpenDF(&fileSpec, fsRdWrPerm, &fRefNum);
  if (error != noErr)
  {
    // throw exception
  }
}

RS_CFile::FileError RS_CFile::Close()
{
FileError error = FSClose(fRefNum);
if (error == noErr)
  fRefNum = kFileError;
  return error;
}

RS_CFile::FileError RS_CFile::GetFileSize(long &fileSize)
{
  FileError error = SetFilePosition(0L);
  if (error == noErr)
    error = GetEOF(fRefNum, &fileSize);
  return error;
}
#endif

RS_CFile::~RS_CFile()
{
  if (IsOpen())
    Close();
}


