// RSFILES.H

#ifndef __RSFILES_H
#define __RSFILES_H

#ifdef RS_BUILD_WIN
#include <windows.h>
#endif

#ifdef RS_BUILD_MAC
#include <files.h>
#endif

#if defined(RS_BUILD_WIN) && !(defined(RS_BUILD_WIN16) || defined(RS_BUILD_WIN32))
#define RS_BUILD_WIN16
#endif

#if defined(RS_BUILD_WIN32) && !defined(RS_BUILD_WIN)
#define RS_BUILD_WIN
#endif

class RS_CFile
{
public:
  typedef short FileError;
  enum {kFileError = -1, kNoError = 0};
#ifdef RS_BUILD_WIN 
  RS_CFile(LPCSTR fileName, int accessMode = OF_READWRITE);
#endif
#ifdef RS_BUILD_MAC
  RS_CFile(FSSpec &fileSpec);
#endif
  ~RS_CFile();
  FileError Read(long &count, void *buffer);
  FileError Write(long &count, void *buffer);
  FileError GetFileSize(long &fileSize);
  FileError SetFilePosition(long filePosition);
  FileError GetFilePosition(long &filePosition);
  int IsOpen();
#ifdef RS_BUILD_WIN
  HFILE GetHFile();
#endif
protected:
  FileError Close();
private:
#ifdef RS_BUILD_WIN
  HFILE fFile;
#endif
#ifdef RS_BUILD_MAC
  short fRefNum;
  FSSpec &fFSSpec;
#endif
};

inline int RS_CFile::IsOpen()
{
#ifdef RS_BUILD_WIN
  return (fFile != kFileError);
#endif
#ifdef RS_BUILD_MAC
  return (fRefNum != kFileError);
#endif
}
#ifdef RS_BUILD_WIN
inline HFILE RS_CFile::GetHFile()
{
  return fFile;
}
#endif
#ifdef RS_BUILD_MAC
inline RS_CFile::FileError RS_CFile::GetFilePosition(long &filePosition)
{
  return GetFPos(fRefNum, &filePosition);
}
inline RS_CFile::FileError RS_CFile::SetFilePosition(long filePosition)
{
  return SetFPos(fRefNum, fsFromStart, filePosition);
}
inline RS_CFile::FileError RS_CFile::Read(long &count, void *buffer)
{
  return FSRead(fRefNum, &count, buffer);
}
inline RS_CFile::FileError RS_CFile::Write(long &count, void *buffer)
{
  return FSWrite(fRefNum, &count, buffer);
}
#endif

#endif //__RSFILES_H


